#
# vocalips030.tcl - GUI
# 
# Rev.0.30  Dec. 24, 2007  add BPM-map preview.
# Rev.0.21  Nov. 27, 2007  just for the update to vsqtxt2lipsync_042.
# Rev.0.20  Nov. 23, 2007
# Rev.0.10  Nov. 3, 2007
# 
# author: zhuo   www.geocities.jp/zhuoware
# 

set VocalipsRevision "0.30"
set Vsqtxt2lipsyncScript "bin/vsqtxt2lipsync_043.pl"

set w .top
catch {destroy $w}
toplevel $w
wm title $w "VOCALIPS Rev.$VocalipsRevision"
wm iconname $w "VOCALIPS Rev.$VocalipsRevision"
wm withdraw .

proc createFileEntry { path text cmd } {
    frame $path
    label $path.lab -width 16 -text $text -anchor e
    entry $path.ent -width 80
    button $path.but -text "Browse ..." -command "$cmd $path"
    pack $path.lab -side left
    pack $path.ent -side left -expand yes -fill x
    pack $path.but -side left
    pack $path -fill x -padx 1c -pady 3
}

proc setEntryString { path string } {
    if {[string compare $string ""]} {
	$path delete 0 end
	$path insert 0 $string
	$path xview end
    }
}

proc consPuts { str } {
    global w
    catch {$w.cons.text insert end $str}
}
if { 0 } {
    frame $w.cons
    scrollbar $w.cons.scroll -command "$w.cons.text yview"
    text $w.cons.text -height 10 -yscroll "$w.cons.scroll set"
    pack $w.cons.scroll -side right -expand no -fill y
    pack $w.cons.text -side left -expand yes -fill both
    pack $w.cons -side bottom  -expand yes -fill both
}




createFileEntry $w.vsqfile "vocaloid file: " "vsqFileChooser $w.outfile.ent"
createFileEntry $w.outfile "output file: " "outFileChooser"
createFileEntry $w.tblfile "phoneme conv. file: " "tblFileChooser"

$w.tblfile.ent delete 0 end
$w.tblfile.ent insert 0 [lindex [glob -nocomplain -types f "*.tbl"] 0] 
$w.tblfile.ent xview end


labelframe $w.bpmmap -text "bpm map" -padx 5 -pady 5
scrollbar $w.bpmmap.scroll -command "$w.bpmmap.list yview"
listbox $w.bpmmap.list -yscroll "$w.bpmmap.scroll set" -setgrid 1 -height 8 -selectmode single
pack $w.bpmmap.scroll -side right -fill y
pack $w.bpmmap.list -side left -expand yes -fill both 
pack $w.bpmmap -side left -expand yes -fill both  -padx 3m


frame $w.settings
labelframe $w.settings.tracks -text "track chooser" -padx 5 -pady 5
scrollbar $w.settings.tracks.scroll -command "$w.settings.tracks.list yview"
listbox $w.settings.tracks.list -yscroll "$w.settings.tracks.scroll set" -setgrid 1 -height 8 -selectmode extended
pack $w.settings.tracks.scroll -side right -fill y
pack $w.settings.tracks.list -side left -expand yes -fill both 
pack $w.settings.tracks -side left -expand yes -fill both  -padx 3m


labelframe $w.settings.mode -text "output mode" -padx 5 -pady 5
radiobutton $w.settings.mode.e2 -text "standard" -variable outputMode -relief flat -value "e2" -anchor w
radiobutton $w.settings.mode.e1 -text "event-base, \"sil\" when early" -variable outputMode -relief flat -value "e1" -anchor w
radiobutton $w.settings.mode.e0 -text "event-base, always with \"end\"" -variable outputMode -relief flat -value "e0" -anchor w
radiobutton $w.settings.mode.f -text "frame-base" -variable outputMode -relief flat -value "f" -anchor w
radiobutton $w.settings.mode.v -text "raw vsq text" -variable outputMode -relief flat -value "v" -anchor w
pack $w.settings.mode.e2 $w.settings.mode.e1 $w.settings.mode.e0 $w.settings.mode.f $w.settings.mode.v -side top -pady 2 -expand no -fill x
pack $w.settings.mode -side left -expand no -fill y  -padx 3m
$w.settings.mode.e2 invoke

labelframe $w.settings.fb -text "options for frame-base mode" -padx 5 -pady 5
label $w.settings.fb.fpslab -text "fps"
entry $w.settings.fb.fpsent -width 8
$w.settings.fb.fpsent insert 0 "30.00"
label $w.settings.fb.offsetlab -text "offset(msec)"
entry $w.settings.fb.offsetent -width 8
$w.settings.fb.offsetent insert 0 "0.0"
pack $w.settings.fb.fpslab $w.settings.fb.fpsent $w.settings.fb.offsetlab $w.settings.fb.offsetent -side left -expand yes -padx 2m
pack $w.settings.fb -side bottom -fill none -padx 3m

pack $w.settings -side top -expand yes -fill both


frame $w.buttons
button $w.buttons.convert -width 16 -text "convert" -command "convert $w"
button $w.buttons.quit -width 16 -text quit -command "destroy $w; exit"
pack $w.buttons.convert -side left -expand 1
pack $w.buttons.quit -side left -expand 1
pack $w.buttons -side bottom -fill x -pady 5m


proc getSelectedTracks {} {
    global w
    return [$w.settings.tracks.list curselection]
}



proc setTrackList {trackListFile} {
    global w Vsqtxt2lipsyncScript
    set listpath $w.settings.tracks.list

    set fd [open "$trackListFile"]
    $listpath delete 0 end
    while {[gets $fd line] >= 0 } {
	$listpath insert end $line	
    }
    close $fd
    $listpath selection set 0 end
}


proc setBPMMap {bpmFile} {
    global w Vsqtxt2lipsyncScript
    set listpath $w.bpmmap.list

    set fd [open "$bpmFile"]
    set isSection 0
   
    $listpath delete 0 end
    while {[gets $fd line] >= 0 } {
	set elms [split $line "\t"]
	if { [lindex $elms 0] == "\#\#" && [lindex $elms 2] == "bpm_map" } {
	    if { [lindex $elms 1] == "begin" } {
		set isSection 1
		continue
	    } elseif { [lindex $elms 1] == "end" } {
		set isSection 0
		continue
	    }
	}
	if { $isSection == 1 } {
	    $listpath insert end $line	
	}
    }
    close $fd
}



proc vsqFileChooser { out_ent path } {
    global Vsqtxt2lipsyncScript
    set types {{"vocaloid files" {.vsq}} {"All files" *}}
    set file "[tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
    if { "$file" == "" || ! [file exists "$file"] } {
	tk_messageBox -icon info -message "Please choose a vocaloid file." -type ok -parent $w
	return
    }
    regsub {\.vsq$} "$file" ".txt" outfile 
    setEntryString $path.ent "$file"
    setEntryString $out_ent "$outfile"

    file copy -force "$file" "T.VSQ"
    if { [catch {exec bin/mf2t "T.VSQ" "T.MFT"} msg ] } {
	tk_messageBox -icon info -message "mf2t error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT"}
	return
    }
    if { [catch {exec bin/Perl $Vsqtxt2lipsyncScript -m t < "T.MFT" > "T1.TXT"} msg ] } {
	tk_messageBox -icon info -message "track-info error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT" "T1.TXT"}
	return
    }
    setTrackList "T1.TXT"
    if { [catch {exec bin/Perl $Vsqtxt2lipsyncScript -m m < "T.MFT" > "T2.TXT"} msg] } {
	tk_messageBox -icon info -message "bpm-info error: $msg" -type ok -parent $w
	catch {file delete "T.VSQ" "T.MFT" "T1.TXT" "T2.TXT"}
	return
    }
    setBPMMap "T2.TXT"
    file delete "T.VSQ" "T.MFT" "T1.TXT" "T2.TXT"
}

proc tblFileChooser { path } {
    set types {{"table files" {.tbl}} {"All files" *}}
    setEntryString $path.ent "[tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
}

proc outFileChooser { path } {
    set types {{"text files" {.txt}} {"All files" *}}
    setEntryString $path.ent "[tk_getSaveFile -filetypes $types -parent $path -initialfile [$path.ent get]]"
}



proc convert { w } {
    global outputMode Vsqtxt2lipsyncScript
    set vsqfile "[$w.vsqfile.ent get]"
    set tblfile "[$w.tblfile.ent get]"
    set outfile "[$w.outfile.ent get]"
    set fps [$w.settings.fb.fpsent get]
    set offset [$w.settings.fb.offsetent get]
    set tracks [join [getSelectedTracks] ","]
    if { "$vsqfile" == "" || ! [file exists "$vsqfile"] } {
	tk_messageBox -icon info -message "Please choose a vocaloid file." -type ok -parent $w
	return
    }
    if { "$tblfile" == "" || ! [file exists "$tblfile"] } {
	tk_messageBox -icon info -message "Please choose a table file." -type ok -parent $w
	return
    }
    if { "$outfile" == "" } {
	tk_messageBox -icon info -message "Please choose an output file." -type ok -parent $w
	return
    }
    if { $outputMode == "" } {
	tk_messageBox -icon info -message "Please choose an output mode." -type ok -parent $w
	return
    }

    if { (![file exists "$outfile"]) || ([tk_messageBox -icon question -message "file exists: overwrite? $outfile" -type okcancel -parent $w] == "ok") } {
	file copy -force "$vsqfile" "T.VSQ"
	exec bin/mf2t "T.VSQ" | bin/Perl $Vsqtxt2lipsyncScript -t "$tracks" -m "$outputMode" -p "$tblfile" "$fps" "$offset" > "T.TXT"

	file copy -force "T.TXT" "$outfile"
	file delete "T.VSQ" "T.TXT"
	tk_messageBox -icon info -message "$outfile created.\n" -type ok -parent $w
    } else {
	tk_messageBox -icon info -message "save canceled.\n" -type ok -parent $w
    }
}

